iceoryx2::node

Enum NodeState

source
pub enum NodeState<Service: Service> {
    Alive(AliveNodeView<Service>),
    Dead(DeadNodeView<Service>),
    Inaccessible(NodeId),
    Undefined(NodeId),
}
The current state of the Node. If the Node is dead all of its resources can be removed with DeadNodeView::remove_stale_resources().

Variants



Alive(AliveNodeView<Service>)

The Nodes process is still alive.



Dead(DeadNodeView<Service>)

The Nodes process died without cleaning up the Nodes resources. Another process has now the responsibility to cleanup all the stale resources.



Inaccessible(NodeId)

The process does not have sufficient permissions to identify the Node as dead or alive.



Undefined(NodeId)

The Node is in an undefined state, meaning that certain elements are missing, misconfigured or inconsistent. This can only happen due to an implementation failure or when the corresponding Node resources were altered.

Implementations

source

impl<Service: Service> NodeState<Service>

source

pub fn node_id(&self) -> &NodeId

Returns the NodeId of the corresponding Node.