NodeBuilder

Struct NodeBuilder 

pub struct NodeBuilder { /* private fields */ }
Creates a Node.

use iceoryx2::prelude::*;

let node = NodeBuilder::new()
                .name(&"my_little_node".try_into()?)
                .create::<ipc::Service>()?;

// do things with your cool new node

Implementations

Source

impl NodeBuilder

pub fn new() -> Self

Creates a new NodeBuilder

pub fn name(self, value: &NodeName) -> Self

Sets the NodeName of the to be created Node.

pub fn signal_handling_mode(self, value: SignalHandlingMode) -> Self

Defines the SignalHandlingMode for the Node. It affects the Node::wait() call that returns any received Signal via its NodeWaitFailure

pub fn config(self, value: &Config) -> Self

Sets the config of the Node that will be used to create all entities owned by the Node.

pub fn create<Service: Service>( self, ) -> Result<Node<Service>, NodeCreationFailure>

Creates a new Node for a specific service::Service. All entities owned by the Node will have the same service::Service.