pub enum EntryHandleError {
EntryDoesNotExist,
}Expand description
Defines a failure that can occur when a EntryHandle is created with Reader::entry().
Variants
EntryDoesNotExist
The entry with the given key and value type does not exist.