#[non_exhaustive]#[repr(C)]pub enum MessagingPattern {
RequestResponse(StaticConfig),
PublishSubscribe(StaticConfig),
Event(StaticConfig),
Blackboard(StaticConfig),
}Expand description
Contains the static config of the corresponding
service::MessagingPattern.
Safety
This is a large struct (>1KB). Be cautious with where it is placed and how it is passed around.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
RequestResponse(StaticConfig)
Stores the static config of the
service::MessagingPattern::RequestResponse
PublishSubscribe(StaticConfig)
Stores the static config of the
service::MessagingPattern::PublishSubscribe
Event(StaticConfig)
Stores the static config of the
service::MessagingPattern::Event
Blackboard(StaticConfig)
Stores the static config of the
service::MessagingPattern::Blackboard
Implementations
Sourceimpl MessagingPattern
impl MessagingPattern
Sourcepub unsafe fn publish_subscribe(&self) -> &StaticConfig
pub unsafe fn publish_subscribe(&self) -> &StaticConfig
Safety
- User must ensure that publish subscribe is stored inside
Sourcepub unsafe fn request_response(&self) -> &StaticConfig
pub unsafe fn request_response(&self) -> &StaticConfig
Safety
- User must ensure that event is stored inside