Module signal_handling_mode

Module signal_handling_mode 

Defines how constructs like the Node or the WaitSet shall handle system signals.

Enums

SignalHandlingMode
Defines how signals are handled by constructs that might register a custom SignalHandler