pub enum NodeState<Service: Service> {
Alive(AliveNodeView<Service>),
Dead(DeadNodeView<Service>),
Inaccessible(NodeId),
Undefined(NodeId),
}Expand description
The current state of the Node. If the Node is dead all of its resources can be removed
with DeadNodeView::remove_stale_resources().
Variants
Alive(AliveNodeView<Service>)
The Nodes process is still alive.
Dead(DeadNodeView<Service>)
The Nodes process died without cleaning up the Nodes resources. Another process has
now the responsibility to cleanup all the stale resources.
Inaccessible(NodeId)
The process does not have sufficient permissions to identify the Node as dead or alive.
Undefined(NodeId)
Implementations