pub struct DeadNodeView<Service: Service>(/* private fields */);Expand description
All the information and management operations belonging to a dead Node.
Implementations
Sourceimpl<Service: Service> DeadNodeView<Service>
impl<Service: Service> DeadNodeView<Service>
Sourcepub fn blocking_remove_stale_resources(
self,
timeout: Duration,
) -> Result<(), NodeCleanupFailure>
pub fn blocking_remove_stale_resources( self, timeout: Duration, ) -> Result<(), NodeCleanupFailure>
Removes all stale resources of a dead Node. If another instance
is already removing the dead Node it waits until the other instance
has cleaned up the dead Node completely. If the other cleanup instance
crashes, it will take over the ownership and continue the cleanup.
If the process does not have the permission to cleanup all resources it
aborts with an error.
If the provided timeout is expired it will return with
NodeCleanupFailure::AnotherInstanceIsCleaningUpTheNode.
Sourcepub fn try_remove_stale_resources(self) -> Result<(), NodeCleanupFailure>
pub fn try_remove_stale_resources(self) -> Result<(), NodeCleanupFailure>