pub enum PublisherCreateError {
ExceedsMaxSupportedPublishers,
UnableToCreateDataSegment,
FailedToDeployThreadsafetyPolicy,
UnableToCreatePortTag,
UnableToGenerateUniquePublisherId,
}Expand description
Defines a failure that can occur when a Publisher is created with
crate::service::port_factory::publisher::PortFactoryPublisher.
Variants
ExceedsMaxSupportedPublishers
The maximum amount of Publishers that can connect to a
Service is
defined in crate::config::Config. When this is exceeded no more Publishers
can be created for a specific Service.
UnableToCreateDataSegment
The datasegment in which the payload of the Publisher is stored, could not be created.
FailedToDeployThreadsafetyPolicy
Caused by a failure when instantiating a ArcSyncPolicy defined in the
Service as ArcThreadSafetyPolicy.
UnableToCreatePortTag
The tracking port tag, required for cleanup, could not be created.
UnableToGenerateUniquePublisherId
The UniquePublisherId could not be generated.